@@ -22,10 +22,62 @@ from ..utils import (
     match_results_structure
 )
 
+def assign_person_by_area(detections, customer_areas, staff_areas):
+    """
+    Assigns category 'person' detections to 'staff' or 'customer' based on their location in area polygons.
+    Modifies the detection list in-place.
+    Args:
+        detections: List of detection dicts.
+        customer_areas: Dict of area_name -> polygon (list of [x, y]).
+        staff_areas: Dict of area_name -> polygon (list of [x, y]).
+    """
+    # First, collect all person detections and their centers
+    staff_ids = set()
+    for det in detections:
+        if det.get('category') == 'person':
+            bbox = det.get('bbox', det.get('bounding_box', None))
+            if bbox and len(bbox) == 4:
+                center = get_bbox_center(bbox)
+                # If in any staff area, mark as staff
+                for polygon in staff_areas.values():
+                    if point_in_polygon(center, polygon):
+                        det['category'] = 'staff'
+                        staff_ids.add(id(det))
+                        break
+    # All other person detections are customers
+    for det in detections:
+        if det.get('category') == 'person' and id(det) not in staff_ids:
+            det['category'] = 'customer'
 
 class AdvancedCustomerServiceUseCase(BaseProcessor):
-    """Advanced customer service analytics with comprehensive business intelligence and journey tracking."""
-    
+    # --- Chunk tracking for per-chunk analytics ---
+    def _init_chunk_tracking(self):
+        self._chunk_frame_count = 0
+        self._chunk_customer_ids = set()
+        self._chunk_area_customer_ids = defaultdict(set)
+
+    def _update_chunk_tracking(self, customer_detections):
+        for customer in customer_detections:
+            track_id = customer.get('track_id')
+            if track_id is not None:
+                self._chunk_customer_ids.add(track_id)
+                # Find all areas this customer is in (from current_areas or by geometry)
+                if 'current_areas' in customer:
+                    for area in customer['current_areas']:
+                        self._chunk_area_customer_ids[area].add(track_id)
+                else:
+                    # fallback: try to infer from bbox and self.customer_areas
+                    customer_center = get_bbox_center(customer.get('bbox', customer.get('bounding_box', {})))
+                    for area_name, polygon in getattr(self, 'customer_areas', {}).items():
+                        if point_in_polygon(customer_center, polygon):
+                            self._chunk_area_customer_ids[area_name].add(track_id)
+
+    def _maybe_reset_chunk(self):
+        if not hasattr(self, '_chunk_frame_count'):
+            self._init_chunk_tracking()
+        self._chunk_frame_count += 1
+        if self._chunk_frame_count > 10:
+            self._init_chunk_tracking()
     def __init__(self):
         """Initialize advanced customer service use case."""
         super().__init__("advanced_customer_service")
@@ -42,6 +94,13 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
         self.staff_service_count = defaultdict(int)
         self.staff_active_services = {}
         
+        # Persistent unique staff tracking
+        self.global_staff_ids = set()
+        self.global_staff_ids_by_area = defaultdict(set)
+        
+        # Persistent unique customer tracking
+        self.global_customer_ids = set()
+        
         # Analytics
         self.queue_wait_times = defaultdict(list)
         self.service_times = defaultdict(list)
@@ -182,17 +241,20 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
             "confidence_threshold": 0.5,
             "enable_tracking": True,
             "enable_analytics": True,
+            "enable_journey_analysis": True,
+            "enable_queue_analytics": True,
             "staff_categories": ["staff", "employee"],
             "customer_categories": ["customer", "person"],
             "service_proximity_threshold": 100.0,
             "max_service_time": 1800.0,
             "buffer_time": 2.0,
+            "stream_info": {},  
         }
         defaults.update(overrides)
         return CustomerServiceConfig(**defaults)
     
     def process(self, data: Any, config: ConfigProtocol,
-                context: Optional[ProcessingContext] = None) -> ProcessingResult:
+                context: Optional[ProcessingContext] = None, stream_info: Optional[dict] = None) -> ProcessingResult:
         """
         Process advanced customer service analytics.
         
@@ -215,60 +277,78 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
                     category=self.category,
                     context=context
                 )
-            
+
+            # Attach stream_info to context if provided
+            if stream_info is not None:
+                if context is None:
+                    context = ProcessingContext()
+                context.stream_info = stream_info
+
             # Initialize processing context if not provided
             if context is None:
                 context = ProcessingContext()
-            
+
+            # Store service_proximity_threshold for use in instance methods
+            self._service_proximity_threshold = config.service_proximity_threshold
+
             # Detect input format
             input_format = match_results_structure(data)
             context.input_format = input_format
             context.confidence_threshold = config.confidence_threshold
             context.enable_tracking = config.enable_tracking
-            
+
             self.logger.info(f"Processing advanced customer service with format: {input_format.value}")
-            
+
             # Initialize area tracking
             self._initialize_areas(config.customer_areas, config.staff_areas, config.service_areas)
-            
+
             # Step 1: Apply confidence filtering
             processed_data = data
             if config.confidence_threshold is not None:
                 processed_data = filter_by_confidence(processed_data, config.confidence_threshold)
                 self.logger.debug(f"Applied confidence filtering with threshold {config.confidence_threshold}")
-            
+
             # Step 2: Apply category mapping if provided
             if hasattr(config, 'index_to_category') and config.index_to_category:
                 processed_data = apply_category_mapping(processed_data, config.index_to_category)
                 self.logger.debug("Applied category mapping")
-            
-            # Step 3: Extract and categorize detections
+
+            # Step 3: Extract detections and assign 'person' by area if needed
             detections = self._extract_detections(processed_data)
+            assign_person_by_area(
+                detections,
+                getattr(config, 'customer_areas', {}),
+                getattr(config, 'staff_areas', {})
+            )
             staff_detections, customer_detections = self._categorize_detections(
                 detections, config.staff_categories, config.customer_categories
             )
-            
             self.logger.debug(f"Extracted {len(staff_detections)} staff and {len(customer_detections)} customer detections")
-            
+
+
+            # --- Chunk tracking logic ---
+            self._maybe_reset_chunk()
+            self._update_chunk_tracking(customer_detections)
+
             # Step 4: Process comprehensive analytics
             current_time = time.time()
             analytics_results = self._process_comprehensive_analytics(
                 staff_detections, customer_detections, config, current_time
             )
-            
+
             # Step 5: Generate insights and alerts
             insights = self._generate_insights(analytics_results, config)
             alerts = self._check_alerts(analytics_results, config)
-            
+
             # Step 6: Generate human-readable summary
             summary = self._generate_summary(analytics_results, alerts)
-            
+
             # Step 7: Extract predictions for API compatibility
             predictions = self._extract_predictions(processed_data)
-            
+
             # Mark processing as completed
             context.mark_completed()
-            
+
             # Create successful result
             result = self.create_result(
                 data=analytics_results,
@@ -276,29 +356,29 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
                 category=self.category,
                 context=context
             )
-            
+
             # Add human-readable information
             result.summary = summary
             result.insights = insights
             result.predictions = predictions
             result.metrics = analytics_results.get("business_metrics", {})
-            
+
             # Add warnings for configuration issues
             if not config.customer_areas and not config.staff_areas:
                 result.add_warning("No customer or staff areas defined - using global analysis only")
-            
+
             if config.service_proximity_threshold > 200:
                 result.add_warning(f"High service proximity threshold ({config.service_proximity_threshold}) may miss interactions")
-            
+
             self.logger.info(f"Advanced customer service analysis completed successfully in {result.processing_time:.2f}s")
             return result
-            
+
         except Exception as e:
             self.logger.error(f"Advanced customer service analysis failed: {str(e)}", exc_info=True)
-            
+
             if context:
                 context.mark_completed()
-            
+
             return self.create_error_result(
                 str(e),
                 type(e).__name__,
@@ -390,10 +470,10 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
             staff_center = get_bbox_center(staff.get('bbox', staff.get('bounding_box', {})))
             if not staff_center:
                 continue
-            
             track_id = staff.get('track_id', f"staff_{hash(str(staff_center))}")
-            
-            # Update staff area occupancy
+            # Update persistent global staff ids
+            self.global_staff_ids.add(track_id)
+            # Update staff area occupancy and persistent area staff ids
             for area_name, polygon in self.staff_areas.items():
                 if point_in_polygon(staff_center, polygon):
                     self.staff_occupancy[area_name].append({
@@ -401,6 +481,7 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
                         'center': staff_center,
                         'timestamp': current_time
                     })
+                    self.global_staff_ids_by_area[area_name].add(track_id)
     
     def _process_customer_detections(self, customer_detections: List[Dict], current_time: float):
         """Process customer detections and update journey tracking."""
@@ -408,9 +489,9 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
             customer_center = get_bbox_center(customer.get('bbox', customer.get('bounding_box', {})))
             if not customer_center:
                 continue
-            
             track_id = customer.get('track_id', f"customer_{hash(str(customer_center))}")
-            
+            # Update persistent global customer ids
+            self.global_customer_ids.add(track_id)
             # Initialize customer journey if new
             if track_id not in self.customer_journey:
                 self._initialize_customer_journey(track_id, current_time)
@@ -491,15 +572,15 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
         customer_journey = self.customer_journey.get(customer_track_id)
         if not customer_journey or not customer_journey['positions']:
             return False
-        
+
         customer_center = customer_journey['positions'][-1]['center']
-        
+
         # Find nearest staff
         nearest_staff = self._find_nearest_staff(customer_center)
         if nearest_staff:
             staff_id, distance = nearest_staff
-            return distance <= self.service_proximity_threshold
-        
+            return distance <= self._service_proximity_threshold
+
         return False
     
     def _find_nearest_staff(self, customer_center: Tuple[float, float]) -> Optional[Tuple[int, float]]:
@@ -542,64 +623,95 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
     
     def _compile_analytics_results(self, current_time: float) -> Dict[str, Any]:
         """Compile comprehensive analytics results."""
+        # --- Previous approach (commented out): ---
+        # real_time_occupancy = {
+        #     "customer_areas": self.customer_occupancy,
+        #     "staff_areas": self.staff_occupancy,
+        #     "service_areas": self.service_occupancy
+        # }
+
+        # --- New approach: Only keep the last detection per track_id per area ---
+        def get_latest_per_track(area_dict):
+            latest = {}
+            for area_name, occupants in area_dict.items():
+                track_map = {}
+                for occ in occupants:
+                    tid = occ.get('track_id')
+                    ts = occ.get('timestamp', 0)
+                    if tid is not None:
+                        if tid not in track_map or ts > track_map[tid]['timestamp']:
+                            track_map[tid] = occ
+                latest[area_name] = list(track_map.values())
+            return latest
+
+        real_time_occupancy = {
+            "customer_areas": get_latest_per_track(self.customer_occupancy),
+            "staff_areas": get_latest_per_track(self.staff_occupancy),
+            "service_areas": get_latest_per_track(self.service_occupancy)
+        }
+
         return {
             "customer_queue_analytics": self._get_customer_queue_results(),
             "staff_management_analytics": self._get_staff_management_results(),
             "service_area_analytics": self._get_service_area_results(),
             "customer_journey_analytics": self._get_customer_journey_results(),
             "business_metrics": self._calculate_analytics(current_time),
-            "real_time_occupancy": {
-                "customer_areas": self.customer_occupancy,
-                "staff_areas": self.staff_occupancy,
-                "service_areas": self.service_occupancy
-            },
+            "real_time_occupancy": real_time_occupancy,
             "processing_timestamp": current_time
         }
     
     def _get_customer_queue_results(self) -> Dict[str, Any]:
-        """Get customer queue analytics."""
-        queue_analytics = {
-            "active_customers": len(self.customer_journey),
-            "customers_queuing": 0,
-            "customers_being_served": 0,
-            "customers_completed": 0,
-            "average_wait_time": 0.0,
-            "queue_lengths_by_area": {}
-        }
-        
+        """Get customer queue analytics (per chunk of 10 frames)."""
+        # Use chunk-based customer ids for per-chunk analytics
+        active_customers = len(getattr(self, '_chunk_customer_ids', set()))
+        queue_lengths_by_area = {}
+        for area_name in self.customer_occupancy:
+            queue_lengths_by_area[area_name] = len(getattr(self, '_chunk_area_customer_ids', defaultdict(set))[area_name])
+
+        # For state counts, only count journeys whose track_id is in the current chunk
+        customers_queuing = 0
+        customers_being_served = 0
+        customers_completed = 0
         wait_times = []
-        for journey in self.customer_journey.values():
+        chunk_ids = getattr(self, '_chunk_customer_ids', set())
+        for track_id in chunk_ids:
+            journey = self.customer_journey.get(track_id)
+            if not journey:
+                continue
             if journey['state'] == self.JOURNEY_STATES['QUEUING']:
-                queue_analytics["customers_queuing"] += 1
+                customers_queuing += 1
             elif journey['state'] == self.JOURNEY_STATES['BEING_SERVED']:
-                queue_analytics["customers_being_served"] += 1
+                customers_being_served += 1
             elif journey['state'] == self.JOURNEY_STATES['COMPLETED']:
-                queue_analytics["customers_completed"] += 1
+                customers_completed += 1
                 if journey['total_wait_time'] > 0:
                     wait_times.append(journey['total_wait_time'])
-        
-        if wait_times:
-            queue_analytics["average_wait_time"] = sum(wait_times) / len(wait_times)
-        
-        # Calculate queue lengths by area
-        for area_name, occupants in self.customer_occupancy.items():
-            queue_analytics["queue_lengths_by_area"][area_name] = len(occupants)
-        
+
+        queue_analytics = {
+            "active_customers": active_customers,
+            "customers_queuing": customers_queuing,
+            "customers_being_served": customers_being_served,
+            "customers_completed": customers_completed,
+            "average_wait_time": sum(wait_times) / len(wait_times) if wait_times else 0.0,
+            "queue_lengths_by_area": queue_lengths_by_area
+        }
         return queue_analytics
     
     def _get_staff_management_results(self) -> Dict[str, Any]:
         """Get staff management analytics."""
+        # Previous (non-persistent) logic:
+        # staff_analytics = {
+        #     "total_staff": sum(len(staff_list) for staff_list in self.staff_occupancy.values()),
+        #     "staff_distribution": {area_name: len(staff_list) ...}
+        # }
+        # Persistent unique logic:
         staff_analytics = {
-            "total_staff": sum(len(staff_list) for staff_list in self.staff_occupancy.values()),
-            "staff_distribution": {},
+            "total_staff": len(self.global_staff_ids),
+            "staff_distribution": {area_name: len(self.global_staff_ids_by_area[area_name]) for area_name in self.staff_areas},
             "staff_efficiency": {},
             "staff_utilization": 0.0
         }
         
-        # Calculate staff distribution
-        for area_name, staff_list in self.staff_occupancy.items():
-            staff_analytics["staff_distribution"][area_name] = len(staff_list)
-        
         # Calculate staff efficiency
         total_services = sum(self.staff_service_count.values())
         active_staff = len(self.staff_service_count)
@@ -902,4 +1014,4 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
         except Exception as e:
             self.logger.warning(f"Failed to extract predictions: {str(e)}")
         
-        return predictions 
\ No newline at end of file
+        return predictions
\ No newline at end of file
